@@ -2,7 +2,7 @@
 """
 Created on Thu Jan 05 13:29:12 2017
 
-@author: Suhas Somnath, Chris R. Smith
+@author: Chris R. Smith, Suhas Somnath
 """
 from __future__ import division, print_function, absolute_import, unicode_literals
 
@@ -15,13 +15,14 @@ from IPython.display import display
 from matplotlib import pyplot as plt
 from functools import partial
 
-from .plot_utils import plot_loops, plot_map_stack, get_cmap_object, plot_map, set_tick_font_size, plot_complex_loop_stack, plot_complex_map_stack
-from pycroscopy.viz.jupyter_utils import save_fig_filebox_button
+from ..core.viz.plot_utils import plot_curves, plot_map_stack, get_cmap_object, plot_map, set_tick_font_size, \
+    plot_complex_spectra
+from ..core.viz.jupyter_utils import save_fig_filebox_button
 from ..analysis.utils.be_loop import loop_fit_function
 from ..analysis.utils.be_sho import SHOfunc
-from ..io.hdf_utils import reshape_to_Ndims, getAuxData, get_sort_order, get_dimensionality, get_attr, \
-    get_source_dataset
-from ..io.pycro_data import PycroDataset
+from ..core.io.hdf_utils import reshape_to_n_dims, get_auxiliary_datasets, get_sort_order, get_dimensionality, \
+    get_attr, get_source_dataset
+from ..core.io.pycro_data import PycroDataset
 
 
 def visualize_sho_results(h5_main, save_plots=True, show_plots=True, cmap=None):
@@ -49,15 +50,15 @@ def visualize_sho_results(h5_main, save_plots=True, show_plots=True, cmap=None):
     def __plot_loops_maps(ac_vec, resp_mat, grp_name, win_title, spec_var_title, meas_var_title, save_plots,
                           folder_path, basename, num_rows, num_cols):
         plt_title = grp_name + '_' + win_title + '_Loops'
-        fig, ax = plot_loops(ac_vec, resp_mat, evenly_spaced=True, plots_on_side=5,
-                             x_label=spec_var_title, y_label=meas_var_title, subtitle_prefix='Position', title=plt_title)
+        fig, ax = plot_curves(ac_vec, resp_mat, evenly_spaced=True, num_plots=25, x_label=spec_var_title,
+                              y_label=meas_var_title, subtitle_prefix='Position', title=plt_title)
         if save_plots:
             fig.savefig(os.path.join(folder_path, basename + '_' + plt_title + '.png'), format='png', dpi=300)
 
         plt_title = grp_name + '_' + win_title + '_Snaps'
         fig, axes = plot_map_stack(resp_mat.reshape(num_rows, num_cols, resp_mat.shape[1]),
-                                   color_bar_mode="each", evenly_spaced=True, title='UDVS Step #',
-                                   heading=plt_title, cmap=cmap)
+                                   color_bar_mode="each", evenly_spaced=True, subtitle='UDVS Step #',
+                                   title=plt_title, cmap=cmap)
         if save_plots:
             fig.savefig(os.path.join(folder_path, basename + '_' + plt_title + '.png'), format='png', dpi=300)
 
@@ -238,12 +239,12 @@ def jupyter_visualize_beps_sho(pc_sho_dset, step_chan, resp_func=None, resp_labe
 
     h5_sho_spec_inds = pc_sho_dset.h5_spec_inds
     h5_sho_spec_vals = pc_sho_dset.h5_spec_vals
-    spec_nd, _ = reshape_to_Ndims(h5_sho_spec_inds, h5_spec=h5_sho_spec_inds)
+    spec_nd, _ = reshape_to_n_dims(h5_sho_spec_inds, h5_spec=h5_sho_spec_inds)
     sho_spec_dims = pc_sho_dset.spec_dim_sizes
     sho_spec_labels = pc_sho_dset.spec_dim_labels
 
     h5_pos_inds = pc_sho_dset.h5_pos_inds
-    pos_nd, _ = reshape_to_Ndims(h5_pos_inds, h5_pos=h5_pos_inds)
+    pos_nd, _ = reshape_to_n_dims(h5_pos_inds, h5_pos=h5_pos_inds)
     pos_dims = pc_sho_dset.pos_dim_sizes
     pos_labels = pc_sho_dset.pos_dim_labels
 
@@ -273,7 +274,7 @@ def jupyter_visualize_beps_sho(pc_sho_dset, step_chan, resp_func=None, resp_labe
     sho_dset_collapsed = np.reshape(guess_nd_data, final_guess_shape).squeeze()
 
     # Get the bias matrix:
-    bias_mat, _ = reshape_to_Ndims(h5_sho_spec_vals, h5_spec=h5_sho_spec_inds)
+    bias_mat, _ = reshape_to_n_dims(h5_sho_spec_vals, h5_spec=h5_sho_spec_inds)
     bias_mat = np.transpose(bias_mat[spec_step_dim_ind],
                             new_spec_order).reshape(sho_dset_collapsed.shape[len(pos_dims):])
     if bias_mat.ndim == 1:
@@ -299,7 +300,8 @@ def jupyter_visualize_beps_sho(pc_sho_dset, step_chan, resp_func=None, resp_labe
             resp_func = ac_spectroscopy_func
             resp_label = 'Amplitude (a. u.)'
 
-    not_step_chan = sho_spec_labels.copy().remove(step_chan)
+    not_step_chan = sho_spec_labels.copy()
+    not_step_chan.remove(step_chan)
     spatial_dict = {step_chan: [step_ind]}
     resp_dict = {pos_labels[-1]: [row_ind],
                  pos_labels[-2]: [col_ind]}
@@ -310,8 +312,8 @@ def jupyter_visualize_beps_sho(pc_sho_dset, step_chan, resp_func=None, resp_labe
         for key in not_step_chan:
             spatial_dict[key] = [0]
 
-    spatial_map = pc_sho_dset.slice(False, spatial_dict)[0][sho_quantity].squeeze()
-    resp_vec = resp_func(pc_sho_dset.slice(False, resp_dict)[0].squeeze())
+    spatial_map = pc_sho_dset.slice(spatial_dict, as_scalar=False)[0][sho_quantity].squeeze()
+    resp_vec = resp_func(pc_sho_dset.slice(resp_dict, as_scalar=False)[0].reshape(bias_mat.shape))
 
     fig = plt.figure(figsize=(12, 8))
     ax_bias = plt.subplot2grid((3, 2), (0, 0), colspan=1, rowspan=1)
@@ -323,7 +325,7 @@ def jupyter_visualize_beps_sho(pc_sho_dset, step_chan, resp_func=None, resp_labe
     ax_bias.set_ylabel(step_chan.replace('_', ' ') + ' (V)')
     bias_slider = ax_bias.axvline(x=step_ind, color='r')
 
-    img_map, img_cmap = plot_map(ax_map, spatial_map.T, show_xy_ticks=None)
+    img_map, img_cmap = plot_map(ax_map, spatial_map.T, show_xy_ticks=True)
 
     map_title = '{} - {}={}'.format(sho_quantity, step_chan, bias_mat[step_ind][0])
     ax_map.set_xlabel(pos_labels[-1])
@@ -350,7 +352,7 @@ def jupyter_visualize_beps_sho(pc_sho_dset, step_chan, resp_func=None, resp_labe
     def update_sho_plots(sho_quantity, step_ind):
         bias_slider.set_xdata((step_ind, step_ind))
         spatial_dict[step_chan] = [step_ind]
-        spatial_map = pc_sho_dset.slice(False, spatial_dict)[0][sho_quantity].squeeze()
+        spatial_map = pc_sho_dset.slice(spatial_dict, as_scalar=False)[0][sho_quantity].squeeze()
         map_title = '{} - {}={}'.format(sho_quantity, step_chan, bias_mat[step_ind][0])
         ax_map.set_title(map_title)
         img_map.set_data(spatial_map.T)
@@ -359,7 +361,7 @@ def jupyter_visualize_beps_sho(pc_sho_dset, step_chan, resp_func=None, resp_labe
         img_map.set_clim(vmin=spat_mean - 3 * spat_std, vmax=spat_mean + 3 * spat_std)
 
     def update_resp_plot(resp_dict):
-        resp_vec = resp_func(np.atleast_2d(pc_sho_dset.slice(False, resp_dict)[0].squeeze()))
+        resp_vec = resp_func(pc_sho_dset.slice(resp_dict, as_scalar=False)[0].reshape(bias_mat.shape)).T
         for line_handle, data in zip(line_handles, resp_vec):
             line_handle.set_ydata(data)
 
@@ -400,7 +402,7 @@ def jupyter_visualize_beps_sho(pc_sho_dset, step_chan, resp_func=None, resp_labe
 
     sho_quantity_picker = widgets.Dropdown(options=list(sho_dset_collapsed.dtype.names[:-1]),
                                            description='SHO Quantity')
-    bias_step_picker = widgets.IntSlider(min=0, max=bias_mat.shape[0]-1, step=1,
+    bias_step_picker = widgets.IntSlider(min=0, max=bias_mat.shape[0] - 1, step=1,
                                          description='Bias Step')
 
     fig_filename, _ = os.path.splitext(pc_sho_dset.file.filename)
@@ -440,10 +442,10 @@ def jupyter_visualize_be_spectrograms(pc_main, cmap=None):
     spec_labels = pc_main.spec_dim_labels
 
     ifreq = spec_labels.index('Frequency')
-    freqs_nd = reshape_to_Ndims(h5_spec_vals, h5_spec=h5_spec_inds)[0][ifreq].squeeze()
+    freqs_nd = reshape_to_n_dims(h5_spec_vals, h5_spec=h5_spec_inds)[0][ifreq].squeeze()
     freqs_2d = freqs_nd.reshape(freqs_nd.shape[0], -1) / 1000  # Convert to kHz
 
-    num_udvs_steps = np.prod([spec_dims[idim] for idim in range(len(spec_dims)) if idim != ifreq])
+    num_udvs_steps = int(np.prod([spec_dims[idim] for idim in range(len(spec_dims)) if idim != ifreq]))
 
     if len(pos_dims) >= 2:
         # Build initial slice dictionaries
@@ -456,16 +458,15 @@ def jupyter_visualize_be_spectrograms(pc_main, cmap=None):
 
         spectrogram_slice_dict = {key: [0] for key in pos_labels}
 
-        spatial_slice, _ = pc_main.get_pos_spec_slices(slice_dict=spatial_slice_dict)
+        spatial_slice, _ = pc_main._get_pos_spec_slices(slice_dict=spatial_slice_dict)
 
         x_size = pos_dims[-1]
         y_size = pos_dims[-2]
 
         spatial_map = np.abs(np.reshape(pc_main[spatial_slice, 0], (y_size, x_size)))
         spectrogram = np.reshape(pc_main[0], (num_udvs_steps, -1))
-        fig, axes = plt.subplots(ncols=3, figsize=(12, 4), subplot_kw={'adjustable': 'box-forced'})
-        spatial_img, spatial_cbar = plot_map(axes[0], np.abs(spatial_map), x_size=x_size,
-                                                         y_size=y_size, cmap=cmap)
+        fig, axes = plt.subplots(ncols=3, figsize=(12, 4), subplot_kw={'adjustable': 'box'})
+        spatial_img, spatial_cbar = plot_map(axes[0], np.abs(spatial_map), cmap=cmap)
         axes[0].set_aspect('equal')
         axes[0].set_xlabel(pos_labels[-1])
         axes[0].set_ylabel(pos_labels[-2])
@@ -475,18 +476,19 @@ def jupyter_visualize_be_spectrograms(pc_main, cmap=None):
         crosshair = axes[0].plot(xdata, ydata, 'k+')[0]
 
         if len(spec_dims) > 1:
-            amp_img, amp_cbar = plot_map(axes[1], np.abs(spectrogram), show_xy_ticks=None, cmap=cmap,
+            amp_img, amp_cbar = plot_map(axes[1], np.abs(spectrogram), show_xy_ticks=True, cmap=cmap,
                                          extent=[freqs_2d[0, 0], freqs_2d[-1, 0], 0, num_udvs_steps])
 
-            phase_img, phase_cbar = plot_map(axes[2], np.angle(spectrogram), show_xy_ticks=None, cmap=cmap,
+            phase_img, phase_cbar = plot_map(axes[2], np.angle(spectrogram), show_xy_ticks=True, cmap=cmap,
                                              extent=[freqs_2d[0, 0], freqs_2d[-1, 0], 0, num_udvs_steps])
+            phase_img.set_clim(vmin=-np.pi, vmax=np.pi)
 
             for axis in axes[1:3]:
                 axis.set_ylabel('BE step')
                 axis.axis('tight')
                 x0, x1 = (freqs_2d[0, 0], freqs_2d[-1, 0])
                 y0, y1 = (0, num_udvs_steps)
-                axis.set_aspect(np.abs(x1-x0)/np.abs(y1-y0))
+                axis.set_aspect(np.abs(x1 - x0) / np.abs(y1 - y0))
 
         else:
             # BE-Line
@@ -517,12 +519,12 @@ def jupyter_visualize_be_spectrograms(pc_main, cmap=None):
         fig.tight_layout()
 
         fig_filename, _ = os.path.splitext(pc_main.file.filename)
-        display(save_fig_filebox_button(fig, fig_filename+'.png'))
+        display(save_fig_filebox_button(fig, fig_filename + '.png'))
 
         # Build sliders for any extra Position Dimensions
         pos_sliders = dict()
         for ikey, key in enumerate(pos_labels[:-2]):
-            pos_sliders[key] = widgets.IntSlider(value=0, min=0, max=pos_dims[ikey]-1,
+            pos_sliders[key] = widgets.IntSlider(value=0, min=0, max=pos_dims[ikey] - 1,
                                                  step=1, description='{} Step:'.format(key),
                                                  continuous_update=False)
 
@@ -533,7 +535,7 @@ def jupyter_visualize_be_spectrograms(pc_main, cmap=None):
             for key in pos_labels[:-2]:
                 spatial_slice_dict[key] = [pos_sliders[key].value]
 
-            spatial_slice, _ = pc_main.get_pos_spec_slices(slice_dict=spatial_slice_dict)
+            spatial_slice, _ = pc_main._get_pos_spec_slices(slice_dict=spatial_slice_dict)
 
             return spatial_slice
 
@@ -543,7 +545,7 @@ def jupyter_visualize_be_spectrograms(pc_main, cmap=None):
             for key in pos_labels[:-2]:
                 spatial_slice_dict[key] = [pos_sliders[key].value]
 
-            spatial_slice, _ = pc_main.get_pos_spec_slices(slice_dict=spatial_slice_dict)
+            spatial_slice, _ = pc_main._get_pos_spec_slices(slice_dict=spatial_slice_dict)
 
             spatial_map = np.abs(np.reshape(pc_main[spatial_slice, step], (x_size, y_size)))
             spatial_img.set_data(spatial_map)
@@ -579,8 +581,12 @@ def jupyter_visualize_be_spectrograms(pc_main, cmap=None):
             spectrogram = np.reshape(pc_main[spatial_slice, :], (num_udvs_steps, -1))
 
             if len(spec_dims) > 1:
+                amp_map = np.abs(spectrogram)
                 amp_img.set_data(np.abs(spectrogram))
                 phase_img.set_data(np.angle(spectrogram))
+                amp_mean = np.mean(amp_map)
+                amp_std = np.std(amp_map)
+                amp_img.set_clim(vmin=amp_mean - 3 * amp_std, vmax=amp_mean + 3 * amp_std)
             else:
                 amp_img.set_ydata(np.abs(spectrogram))
                 phase_img.set_ydata(np.angle(spectrogram))
@@ -615,7 +621,6 @@ def jupyter_visualize_be_spectrograms(pc_main, cmap=None):
 
             fig.canvas.draw()
 
-
         spec_index_slider = widgets.IntSlider(value=0, min=0, max=pc_main.shape[1], step=1,
                                               description='Step')
         cid = spatial_img.figure.canvas.mpl_connect('button_press_event', pos_picker)
@@ -682,7 +687,7 @@ def jupyter_visualize_beps_loops(h5_projected_loops, h5_loop_guess, h5_loop_fit,
     h5_loop_guess : h5py.Dataset
         Dataset holding the loop guesses
     h5_loop_fit : h5py.Dataset
-        Dataset holding the loop gits
+        Dataset holding the loop fits
     step_chan : str, optional
         The name of the Spectroscopic dimension to plot versus.  Needs testing.
         Default 'DC_Offset'
@@ -697,23 +702,23 @@ def jupyter_visualize_beps_loops(h5_projected_loops, h5_loop_guess, h5_loop_fit,
 
     # Prepare some variables for plotting loops fits and guesses
     # Plot the Loop Guess and Fit Results
-    proj_nd, _ = reshape_to_Ndims(h5_projected_loops)
-    guess_nd, _ = reshape_to_Ndims(h5_loop_guess)
-    fit_nd, _ = reshape_to_Ndims(h5_loop_fit)
+    proj_nd, _ = reshape_to_n_dims(h5_projected_loops)
+    guess_nd, _ = reshape_to_n_dims(h5_loop_guess)
+    fit_nd, _ = reshape_to_n_dims(h5_loop_fit)
 
     h5_projected_loops = h5_loop_guess.parent['Projected_Loops']
-    h5_proj_spec_inds = getAuxData(h5_projected_loops,
-                                   auxDataName='Spectroscopic_Indices')[-1]
-    h5_proj_spec_vals = getAuxData(h5_projected_loops,
-                                   auxDataName='Spectroscopic_Values')[-1]
-    h5_pos_inds = getAuxData(h5_projected_loops,
-                             auxDataName='Position_Indices')[-1]
-    pos_nd, _ = reshape_to_Ndims(h5_pos_inds, h5_pos=h5_pos_inds)
+    h5_proj_spec_inds = get_auxiliary_datasets(h5_projected_loops,
+                                               aux_dset_name='Spectroscopic_Indices')[-1]
+    h5_proj_spec_vals = get_auxiliary_datasets(h5_projected_loops,
+                                               aux_dset_name='Spectroscopic_Values')[-1]
+    h5_pos_inds = get_auxiliary_datasets(h5_projected_loops,
+                                         aux_dset_name='Position_Indices')[-1]
+    pos_nd, _ = reshape_to_n_dims(h5_pos_inds, h5_pos=h5_pos_inds)
     pos_dims = list(pos_nd.shape[:h5_pos_inds.shape[1]])
     pos_labels = get_attr(h5_pos_inds, 'labels')
 
     # reshape the vdc_vec into DC_step by Loop
-    spec_nd, _ = reshape_to_Ndims(h5_proj_spec_vals, h5_spec=h5_proj_spec_inds)
+    spec_nd, _ = reshape_to_n_dims(h5_proj_spec_vals, h5_spec=h5_proj_spec_inds)
     loop_spec_dims = np.array(spec_nd.shape[1:])
     loop_spec_labels = get_attr(h5_proj_spec_vals, 'labels')
 
@@ -728,7 +733,7 @@ def jupyter_visualize_beps_loops(h5_projected_loops, h5_loop_guess, h5_loop_fit,
 
     # Also reshape the projected loops to Positions-DC_Step-Loop
     final_loop_shape = pos_dims + [loop_spec_dims[spec_step_dim_ind]] + [-1]
-    proj_nd2 = np.moveaxis(proj_nd, spec_step_dim_ind+len(pos_dims), len(pos_dims))
+    proj_nd2 = np.moveaxis(proj_nd, spec_step_dim_ind + len(pos_dims), len(pos_dims))
     proj_nd_3 = np.reshape(proj_nd2, final_loop_shape)
 
     # Do the same for the guess and fit datasets
@@ -762,8 +767,10 @@ def jupyter_visualize_beps_loops(h5_projected_loops, h5_loop_guess, h5_loop_fit,
     ax_map = plt.subplot2grid((1, 2), (0, 0), colspan=1, rowspan=1)
     ax_loop = plt.subplot2grid((1, 2), (0, 1), colspan=1, rowspan=1)
 
-    im_map, im_cbar = plot_map(ax_map, spatial_map.T, x_size=spatial_map.shape[0],
-                                           y_size=spatial_map.shape[1], cmap=cmap)
+    im_map, im_cbar = plot_map(ax_map, spatial_map.T,
+                               x_vec=np.arange(spatial_map.shape[0]),
+                               y_vec=np.arange(spatial_map.shape[1]),
+                               cmap=cmap)
 
     ax_map.set_xlabel('X')
     ax_map.set_ylabel('Y')
@@ -787,7 +794,7 @@ def jupyter_visualize_beps_loops(h5_projected_loops, h5_loop_guess, h5_loop_fit,
     fig_filename, _ = os.path.splitext(h5_projected_loops.file.filename)
     display(save_fig_filebox_button(fig, fig_filename + '.png'))
 
-    loop_slider = widgets.IntSlider(min=0, max=bias_vec.shape[1]-1, description='Loop Number')
+    loop_slider = widgets.IntSlider(min=0, max=bias_vec.shape[1] - 1, description='Loop Number')
 
     def update_loop_plots(loop_field, loop_ind):
         spatial_map = fit_3d[:, :, loop_ind][loop_field]
@@ -892,7 +899,7 @@ def jupyter_visualize_parameter_maps(h5_loop_parameters, cmap=None, **kwargs):
         map_titles.append(' - '.join(title_list))
 
     fig, axes = plot_map_stack(parameter_map_stack, num_comps=num_cycles, color_bar_mode='each',
-                               title=map_titles, heading='Maps of Loop Parameter {}'.format(parameter_names[0]),
+                               subtitle=map_titles, title='Maps of Loop Parameter {}'.format(parameter_names[0]),
                                **kwargs)
 
     def update_loop_maps(parameter_name):
@@ -939,30 +946,29 @@ def jupyter_visualize_loop_sho_raw_comparison(h5_loop_parameters, cmap=None):
     h5_sho_fit = h5_sho_grp['Fit']
     h5_sho_guess = h5_sho_grp['Guess']
 
-
     h5_main = get_source_dataset(h5_sho_grp)
 
     # Now get the needed ancillary datasets for each main dataset
-    h5_pos_inds = getAuxData(h5_loop_parameters, 'Position_Indices')[0]
-    h5_pos_vals = getAuxData(h5_loop_parameters, 'Position_Values')[0]
+    h5_pos_inds = get_auxiliary_datasets(h5_loop_parameters, 'Position_Indices')[0]
+    h5_pos_vals = get_auxiliary_datasets(h5_loop_parameters, 'Position_Values')[0]
     pos_order = get_sort_order(np.transpose(h5_pos_inds))
     pos_dims = get_dimensionality(np.transpose(h5_pos_inds), pos_order)
     pos_labs = get_attr(h5_pos_inds, 'labels')
 
-    h5_loop_spec_inds = getAuxData(h5_loop_parameters, 'Spectroscopic_Indices')[0]
-    h5_loop_spec_vals = getAuxData(h5_loop_parameters, 'Spectroscopic_Values')[0]
+    h5_loop_spec_inds = get_auxiliary_datasets(h5_loop_parameters, 'Spectroscopic_Indices')[0]
+    h5_loop_spec_vals = get_auxiliary_datasets(h5_loop_parameters, 'Spectroscopic_Values')[0]
     loop_spec_order = get_sort_order(h5_loop_spec_inds)
     loop_spec_dims = get_dimensionality(h5_loop_spec_inds, loop_spec_order)
     loop_spec_labs = get_attr(h5_loop_spec_inds, 'labels')
 
-    h5_sho_spec_inds = getAuxData(h5_sho_fit, 'Spectroscopic_Indices')[0]
-    h5_sho_spec_vals = getAuxData(h5_sho_fit, 'Spectroscopic_Values')[0]
+    h5_sho_spec_inds = h5_sho_fit.h5_spec_inds
+    h5_sho_spec_vals = h5_sho_fit.h5_spec_vals
     sho_spec_order = get_sort_order(h5_sho_spec_inds)
     sho_spec_dims = get_dimensionality(h5_sho_spec_inds, sho_spec_order)
     sho_spec_labs = get_attr(h5_sho_spec_inds, 'labels')
 
-    h5_main_spec_inds = getAuxData(h5_main, 'Spectroscopic_Indices')[0]
-    h5_main_spec_vals = getAuxData(h5_main, 'Spectroscopic_Values')[0]
+    h5_main_spec_inds = h5_main.h5_spec_inds
+    h5_main_spec_vals = h5_main.h5_spec_vals
     main_spec_order = get_sort_order(h5_main_spec_inds)
     main_spec_dims = get_dimensionality(h5_main_spec_inds, main_spec_order)
     main_spec_labs = get_attr(h5_main_spec_inds, 'labels')
@@ -980,8 +986,8 @@ def jupyter_visualize_loop_sho_raw_comparison(h5_loop_parameters, cmap=None):
     selected_loop_parm = loop_parameter_names[0]
     selected_loop_cycle = 0
     selected_loop_ndims = np.unravel_index(selected_loop_cycle, loop_spec_dims, order='F')
-    selected_loop_pos = int(pos_dims[0]/2), int(pos_dims[1]/2)
-    selected_step = int(steps_per_loop/2)
+    selected_loop_pos = int(pos_dims[0] / 2), int(pos_dims[1] / 2)
+    selected_step = int(steps_per_loop / 2)
 
     '''
     Get the bias vector to be plotted against
@@ -996,12 +1002,13 @@ def jupyter_visualize_loop_sho_raw_comparison(h5_loop_parameters, cmap=None):
     Get the frequency vector to be plotted against
     '''
     full_w_vec = h5_main_spec_vals[h5_main_spec_vals.attrs['Frequency']]
-    full_w_vec, _ = reshape_to_Ndims(full_w_vec, h5_spec=h5_main_spec_inds)
+    full_w_vec, _ = reshape_to_n_dims(full_w_vec, h5_spec=h5_main_spec_inds)
     full_w_vec = full_w_vec.squeeze()
 
     '''
     Define functions to get the data
     '''
+
     def _get_loop_map(selected_loop_parm, selected_loop_cycle):
         # Build the map of the chosen loop parameter
         loop_parameter_map = np.reshape(h5_loop_parameters[selected_loop_parm, :, selected_loop_cycle],
@@ -1028,7 +1035,7 @@ def jupyter_visualize_loop_sho_raw_comparison(h5_loop_parameters, cmap=None):
         loop_guess_vec = loop_fit_function(selected_loop_bias_vec,
                                            h5_loop_guess[pos_ind, selected_loop_cycle].tolist())
         loop_fit_vec = loop_fit_function(selected_loop_bias_vec,
-                                           h5_loop_fit[pos_ind, selected_loop_cycle].tolist())
+                                         h5_loop_fit[pos_ind, selected_loop_cycle].tolist())
 
         return selected_loop_bias_vec, loop_proj_vec, loop_guess_vec, loop_fit_vec
 
@@ -1049,7 +1056,7 @@ def jupyter_visualize_loop_sho_raw_comparison(h5_loop_parameters, cmap=None):
         sho_fit = SHOfunc(sho_fit, w_vec)
 
         # Get the slice of the Raw Data
-        raw_data_vec, _ = reshape_to_Ndims(np.atleast_2d(h5_main[pos_ind]), h5_spec=h5_main_spec_inds)
+        raw_data_vec, _ = reshape_to_n_dims(np.atleast_2d(h5_main[pos_ind]), h5_spec=h5_main_spec_inds)
         raw_data_vec = np.moveaxis(raw_data_vec.squeeze(),
                                    main_bias_dim - len(pos_dims), -1)[selected_step][selected_loop_ndims]
 
@@ -1149,8 +1156,10 @@ def jupyter_visualize_loop_sho_raw_comparison(h5_loop_parameters, cmap=None):
     fig_filename, _ = os.path.splitext(h5_loop_parameters.file.filename)
     display(save_fig_filebox_button(fig, fig_filename + '.png'))
     display(fig)
-    x_pos_widget = widgets.FloatSlider(min=0.0, max=float(pos_dims[0]), step=pos_dims[0]/100, value=selected_loop_pos[0])
-    y_pos_widget = widgets.FloatSlider(min=0.0, max=float(pos_dims[1]), step=pos_dims[1] / 100, value=selected_loop_pos[1])
+    x_pos_widget = widgets.FloatSlider(min=0.0, max=float(pos_dims[0]), step=pos_dims[0] / 100,
+                                       value=selected_loop_pos[0])
+    y_pos_widget = widgets.FloatSlider(min=0.0, max=float(pos_dims[1]), step=pos_dims[1] / 100,
+                                       value=selected_loop_pos[1])
     loop_cycle_widget = widgets.IntSlider(min=0, max=loop_num_cycles, step=1, value=selected_loop_cycle)
     spec_step_widget = widgets.IntSlider(min=0, max=steps_per_loop, step=1, value=selected_step)
     widgets.interact(_update_loop_map,
@@ -1170,7 +1179,7 @@ def jupyter_visualize_loop_sho_raw_comparison(h5_loop_parameters, cmap=None):
 
 
 def plot_loop_sho_raw_comparison(h5_loop_parameters, selected_loop_parm=None, selected_loop_cycle=0,
-                                 selected_loop_pos=[0,0], selected_step=0, tick_font_size=14, cmap='viridis',
+                                 selected_loop_pos=[0, 0], selected_step=0, tick_font_size=14, cmap='viridis',
                                  step_chan='DC_Offset'):
     """
 
@@ -1211,9 +1220,8 @@ def plot_loop_sho_raw_comparison(h5_loop_parameters, selected_loop_parm=None, se
     h5_sho_fit = PycroDataset(h5_sho_grp['Fit'], sort_dims=False)
     h5_sho_guess = PycroDataset(h5_sho_grp['Guess'], sort_dims=False)
 
-
     h5_main = get_source_dataset(h5_sho_grp)
-    h5_main.toggle_sorting()
+    # h5_main.toggle_sorting()
 
     # Now get the needed ancillary datasets for each main dataset
     pos_dims = h5_loop_parameters.pos_dim_sizes
@@ -1243,10 +1251,10 @@ def plot_loop_sho_raw_comparison(h5_loop_parameters, selected_loop_parm=None, se
     loop_parameter_names = h5_loop_parameters.dtype.names
     loop_num_cycles = h5_loop_parameters.shape[1]
     # loop_parameter_spec_labs = h5_loop_parameters.spec_dim_labels
-    sho_bias_dim = np.argwhere(np.array(sho_spec_labs) == step_chan).squeeze()
+    sho_bias_dim = sho_spec_labs.index(step_chan)
     steps_per_loop = sho_spec_dims[sho_bias_dim]
-    main_bias_dim = np.argwhere(np.array(main_spec_labs) == step_chan).squeeze()
-    main_freq_dim = np.argwhere(np.array(main_spec_labs) == 'Frequency').squeeze()
+    main_bias_dim = main_spec_labs.index(step_chan)
+    main_freq_dim = main_spec_labs.index('Frequency')
 
     if selected_loop_parm is None:
         selected_loop_parm = loop_parameter_names[0]
@@ -1258,7 +1266,7 @@ def plot_loop_sho_raw_comparison(h5_loop_parameters, selected_loop_parm=None, se
     loop_bias_vec = h5_sho_spec_vals[get_attr(h5_sho_spec_vals, step_chan)].squeeze()
     shift_ind = int(-1 * steps_per_loop / 4)
     loop_bias_vec = loop_bias_vec.reshape(sho_spec_dims[::-1])
-    loop_bias_vec = np.moveaxis(loop_bias_vec, len(loop_bias_vec.shape)-sho_bias_dim-1, 0)
+    loop_bias_vec = np.moveaxis(loop_bias_vec, len(loop_bias_vec.shape) - sho_bias_dim - 1, 0)
     loop_bias_vec = np.reshape(loop_bias_vec, [sho_spec_dims[sho_bias_dim], -1])
     loop_bias_vec = np.roll(loop_bias_vec.reshape(steps_per_loop, -1), shift_ind, axis=0)
 
@@ -1266,12 +1274,13 @@ def plot_loop_sho_raw_comparison(h5_loop_parameters, selected_loop_parm=None, se
     Get the frequency vector to be plotted against
     '''
     full_w_vec = h5_main_spec_vals[h5_main_spec_vals.attrs['Frequency']]
-    full_w_vec, _ = reshape_to_Ndims(full_w_vec, h5_spec=h5_main_spec_inds)
+    full_w_vec, _ = reshape_to_n_dims(full_w_vec, h5_spec=h5_main_spec_inds)
     full_w_vec = full_w_vec.squeeze()
 
     '''
     Define functions to get the data
     '''
+
     def _get_loop_map(selected_loop_parm, selected_loop_cycle):
         # Build the map of the chosen loop parameter
         loop_parameter_map = np.reshape(h5_loop_parameters[selected_loop_parm, :, selected_loop_cycle],
@@ -1288,7 +1297,7 @@ def plot_loop_sho_raw_comparison(h5_loop_parameters, selected_loop_parm=None, se
         return loop_parameter_map, loop_map_title
 
     def _get_loops(selected_loop_cycle, selected_loop_pos):
-        selected_loop_ndims = np.unravel_index(selected_loop_cycle, loop_spec_dims[::-1], order='F')
+        selected_loop_ndims = np.unravel_index(selected_loop_cycle, loop_spec_dims, order='F')
         # Now build the loop plot for the selected position in the loop map
         selected_loop_bias_vec = loop_bias_vec[:, selected_loop_cycle]
 
@@ -1301,54 +1310,44 @@ def plot_loop_sho_raw_comparison(h5_loop_parameters, selected_loop_parm=None, se
         for spec_dim, dim_ind in zip(loop_spec_labs, selected_loop_ndims):
             slice_dict[spec_dim] = [dim_ind]
 
-        loop_proj_vec, _ = h5_loop_projections.slice(False, slice_dict)
+        loop_proj_vec, _ = h5_loop_projections.slice(slice_dict, as_scalar=False)
         loop_proj_vec2 = np.roll(loop_proj_vec.squeeze(), shift_ind)
 
-        loop_guess_slice, _ = h5_loop_guess.slice(True, slice_dict)
-        loop_fit_slice, _ = h5_loop_fit.slice(True, slice_dict)
-
-        # loop_proj_vec = h5_loop_projections[pos_ind].reshape(sho_spec_dims[::-1])
-        # loop_proj_vec2 = np.rollaxis(loop_proj_vec,
-        #                             len(sho_spec_dims)-sho_bias_dim-1,
-        #                             len(sho_spec_dims))[selected_loop_ndims]
-        # loop_proj_vec3 = np.roll(loop_proj_vec2, shift_ind)
+        loop_guess_slice, _ = h5_loop_guess.slice(slice_dict, as_scalar=True)
+        loop_fit_slice, _ = h5_loop_fit.slice(slice_dict, as_scalar=True)
 
         loop_guess_vec = loop_fit_function(selected_loop_bias_vec,
                                            loop_guess_slice.squeeze().tolist())
         loop_fit_vec = loop_fit_function(selected_loop_bias_vec,
-                                           loop_fit_slice.squeeze().tolist())
+                                         loop_fit_slice.squeeze().tolist())
 
         return selected_loop_bias_vec, loop_proj_vec2, loop_guess_vec, loop_fit_vec
 
     def _get_sho(selected_loop_pos, selected_step, selected_loop_cycle):
-        selected_loop_ndims = np.unravel_index(selected_loop_cycle, loop_spec_dims[::-1], order='F')
+        selected_loop_ndims = np.unravel_index(selected_loop_cycle, loop_spec_dims, order='F')
         # get the SHO Guess and Fit and Raw Data for the selected position, cycle, and step
         pos_ind = np.ravel_multi_index(selected_loop_pos, pos_dims)
 
-        # Get the frequency vector for the selected step
-        w_vec = np.rollaxis(full_w_vec, main_freq_dim, len(main_spec_dims))  # Move frequency to the end
-        w_vec2 = np.rollaxis(w_vec, main_bias_dim-1, 0)[selected_step][selected_loop_ndims[::-1]]
-
         # Get the slice of the sho guess and fit
-        sho_guess = h5_sho_guess[pos_ind].reshape(sho_spec_dims[::-1])
-        sho_guess = np.rollaxis(sho_guess,
-                                len(sho_spec_dims)-sho_bias_dim-1,
-                                len(sho_spec_dims))[selected_loop_ndims][selected_step]
-        sho_guess = SHOfunc(sho_guess, w_vec2)
-        sho_fit = h5_sho_fit[pos_ind].reshape(sho_spec_dims[::-1])
-        sho_fit = np.rollaxis(sho_fit,
-                              len(sho_spec_dims)-sho_bias_dim-1,
-                              len(sho_spec_dims))[selected_loop_ndims][selected_step]
-        sho_fit = SHOfunc(sho_fit, w_vec2)
+        sho_slice = {key: int(val) for key, val in zip(pos_labs, selected_loop_pos)}
+        sho_slice[sho_spec_labs[sho_bias_dim]] = selected_step
+        for key, val in zip(loop_spec_labs, selected_loop_ndims):
+            sho_slice[key] = int(val)
 
         # Get the slice of the Raw Data
-        raw_data_vec, _ = reshape_to_Ndims(np.atleast_2d(h5_main[pos_ind]), h5_spec=h5_main_spec_inds)
-        # Move freqency dimension to the end
-        raw_data_vec2 = np.rollaxis(raw_data_vec.squeeze(), main_freq_dim, len(main_spec_dims))
-        # Now move the bias dimension to the front
-        raw_data_vec3 = np.rollaxis(raw_data_vec2, main_bias_dim-1, 0)[selected_step][selected_loop_ndims[::-1]]
+        raw_data_vec = h5_main.slice(sho_slice)[0]
 
-        return w_vec2, sho_guess, sho_fit, raw_data_vec3
+        # Get the frequency vector for the selected step
+        w_vec = np.moveaxis(full_w_vec, main_freq_dim, len(main_spec_dims) - 1)  # Move frequency to the end
+        w_vec2 = np.rollaxis(w_vec, main_bias_dim - 1, 0)[selected_step][selected_loop_ndims]
+
+        sho_guess = h5_sho_guess.slice(sho_slice)[0].tolist()
+
+        sho_guess = SHOfunc(sho_guess, w_vec2)
+        sho_fit = h5_sho_fit.slice(sho_slice)[0].tolist()
+        sho_fit = SHOfunc(sho_fit, w_vec2)
+
+        return w_vec2, sho_guess, sho_fit, raw_data_vec
 
     '''
     Get the starting values
@@ -1592,9 +1591,9 @@ def plot_1d_spectrum(data_vec, freq, title, **kwargs):
     freq *= 1E-3  # to kHz
 
     title = title + ': mean UDVS, mean spatial response'
-    fig, axes = plot_complex_loop_stack(np.expand_dims(data_vec, axis=0), freq, title=title,
-                                        subtitle_prefix='', num_comps=1, x_label='Frequency (kHz)',
-                                        figsize=(5, 3), amp_units='V', **kwargs)
+    fig, axes = plot_complex_spectra(np.expand_dims(data_vec, axis=0), freq, title=title,
+                                     subtitle_prefix='', num_comps=1, x_label='Frequency (kHz)',
+                                     figsize=(5, 3), amp_units='V', **kwargs)
     return fig, axes
 
 
@@ -1624,11 +1623,84 @@ def plot_2d_spectrogram(mean_spectrogram, freq, title=None, **kwargs):
         else:
             raise ValueError('plot_2d_spectrogram: Incompatible data sizes!!!! spectrogram: '
                              + str(mean_spectrogram.shape) + ', frequency: ' + str(freq.shape))
-    freq *= 1E-3  # to kHz
 
-    fig, axes = plot_complex_map_stack(np.expand_dims(mean_spectrogram, axis=0), num_comps=1, title=title,
-                                       x_label='Frequency (kHz)', y_label='UDVS step', subtitle_prefix='',
-                                       extent=[freq[0], freq[-1], 0, mean_spectrogram.shape[0]],
-                                       figsize=(5, 3), origin='lower', stdevs=None, amp_units='V',
-                                       **kwargs)
-    return fig, axes
\ No newline at end of file
+    fig, axes = plot_complex_spectra(np.expand_dims(mean_spectrogram, axis=0), num_comps=1, title=title,
+                                     x_label='Frequency (kHz)', y_label='UDVS step', subtitle_prefix='',
+                                     figsize=(5, 3), origin='lower', stdevs=None, amp_units='V', **kwargs)
+
+    # Changing the X axis labels
+    x_ticks = np.linspace(0, mean_spectrogram.shape[1] - 1, 5, dtype=int)
+    x_tick_labs = [str(np.round(freq[ind], 2)) for ind in x_ticks]
+
+    for axis in axes:
+        axis.set_xticks(x_ticks)
+        axis.set_xticklabels(x_tick_labs)
+
+    return fig, axes
+
+
+def plot_histograms(p_hist, p_hbins, title, figure_path=None):
+    """
+    Plots the position averaged spectrogram
+
+    Parameters
+    ------------
+    p_hist : 2D numpy array
+        histogram data arranged as [physical quantity, frequency bin]
+    p_hbins : 1D numpy array
+        BE frequency that serves as the X axis of the plot
+    title : String
+        Plot group name
+    figure_path : String / Unicode
+        Absolute path of the file to write the figure to
+
+    Returns
+    ---------
+    fig : Matplotlib.pyplot figure
+        Figure handle
+    """
+
+    base_fig_size = 7
+    h_fig = base_fig_size
+    w_fig = base_fig_size * 4
+
+    fig = plt.figure(figsize=(w_fig, h_fig))
+    fig.suptitle(title)
+    iplot = 0
+
+    p_Nx, p_Ny = np.amax(p_hbins, axis=1) + 1
+
+    p_hist = np.reshape(p_hist, (4, p_Ny, p_Nx))
+
+    iplot += 1
+    p_plot_title = 'Spectral BEHistogram Amp (log10 of counts)'
+    p_plot = fig.add_subplot(1, 4, iplot, title=p_plot_title)
+    p_im = p_plot.imshow(np.rot90(np.log10(p_hist[0])), interpolation='nearest')
+    p_plot.axis('tight')
+    fig.colorbar(p_im, fraction=0.1)
+
+    iplot += 1
+    p_plot_title = 'Spectral BEHistogram Phase (log10 of counts)'
+    p_plot = fig.add_subplot(1, 4, iplot, title=p_plot_title)
+    p_im = p_plot.imshow(np.rot90(np.log10(p_hist[1])), interpolation='nearest')
+    p_plot.axis('tight')
+    fig.colorbar(p_im, fraction=0.1)
+
+    iplot += 1
+    p_plot_title = 'Spectral BEHistogram Real (log10 of counts)'
+    p_plot = fig.add_subplot(1, 4, iplot, title=p_plot_title)
+    p_im = p_plot.imshow(np.rot90(np.log10(p_hist[2])), interpolation='nearest')
+    p_plot.axis('tight')
+    fig.colorbar(p_im, fraction=0.1)
+
+    iplot += 1
+    p_plot_title = 'Spectral BEHistogram Imag (log10 of counts)'
+    p_plot = fig.add_subplot(1, 4, iplot, title=p_plot_title)
+    p_im = p_plot.imshow(np.rot90(np.log10(p_hist[3])), interpolation='nearest')
+    p_plot.axis('tight')
+    fig.colorbar(p_im, fraction=0.1)
+
+    if figure_path:
+        plt.savefig(figure_path, format='png')
+
+    return fig
